--[[
Author: Michael Joseph Murray aka Lyte of Lothar(US)
Prescription is a consumables usage check addon designed for end-game raiding
$Date: 2012-11-02 20:53:10 +0000 (Fri, 02 Nov 2012) $
$Revision: 158 $
r158
contact: codemaster2010 AT gmail DOT com

Copyright (c) 2007-2012 Michael J. Murray aka Lyte of Lothar(US)
All rights reserved unless otherwise explicitly stated.
]]

Prescription = LibStub("AceAddon-3.0"):NewAddon("Prescription", "AceTimer-3.0", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Prescription")

local format = string.format
local tinsert = table.insert
local tremove = table.remove
local find = string.find
local pairs = pairs
local strlower = string.lower
local select = select

local UnitName = _G.UnitName
local UnitIsConnected = _G.UnitIsConnected
local UnitIsVisible = _G.UnitIsVisible
local UnitIsDead = _G.UnitIsDead

local user = UnitName("player")

local battleList = {
	[105688] = true, --monk's elixir (750 mastery)
	[105682] = true, --mad hozen elixir (750 crit)
	[105683] = true, --elixir of weaponry (750 expertise)
	[105684] = true, --elixir of rapids (750 haste)
	[105686] = true, --elixir of perfection (750 hit)
	[105685] = true, --elixir of peace (750 spirit)
}

local guardianList = {
	[105681] = true, --mantid elixir (2250 armor)
	[105687] = true, --elixir of mirrors (750 dodge)
}

local flaskList = {
	[105696] = true, --flask of winter's bite (strength)
	[105691] = true, --flask of the warm sun (intellect)
	[105694] = true, --flask of the earth (staminia)
	[105689] = true, --flask of spring blossoms (agility)
	[105693] = true, --flask of falling leaves (spirit)
}

local foodList = {
	[104272] = true, --300 strength
	[104283] = true, --450 stamina
	[104277] = true, --300 intellect
	[104280] = true, --300 spirit
	[104275] = true, --300 agility
}

local foodList275 = {
	[104271] = true, --275 strength
	[104282] = true, --415 stamina
	[104276] = true, --275 intellect
	[104279] = true, --275 spirit
	[104274] = true, --275 agility
}

local foodList250 = {
	[104267] = true, --250 strength
	[104281] = true, --375 stamina
	[104264] = true, --250 intellect
	[104278] = true, --250 spirit
	[104273] = true, --250 agility
}

function Prescription:OnInitialize()
	local defaults = {
		profile = {
			checkBattle = true,
			checkGuardian = true,
			checkFood = true,
			checkFlask = true,
			sendWhispers = false,
			sendRaid = false,
			sendOfficer = false,
			sendSelf = true,
			blockOutgoing = false,
			delay = 15,
			sendCompliance = true,
			sendChannel = false,
			channel = "",
			hideAlert = false,
			allow250StatFood = true,
			allow275StatFood = true,
		},
	}
	
	self.db = LibStub("AceDB-3.0"):New("PrescriptionDB", defaults, "Default")
	
	local opts = {
		type = 'group',
		args = {
			check = {
				type = 'execute',
				name = L["Check Consumables"],
				desc = L["Check the raid for consumable usage"],
				func = function()
					self:ScheduleTimer("Raid_Check", self.db.profile.delay)
					if not self.db.profile.hideAlert then
						if UnitIsGroupLeader("player") or UnitIsGroupAssistant("player") then
							SendChatMessage(format(L["RX Check will be run in %d seconds!"], self.db.profile.delay), "RAID_WARNING")
						else
							SendChatMessage(format(L["RX Check will be run in %d seconds!"], self.db.profile.delay), "RAID")
						end
					end
				end,
			},
			delay = {
				type = 'range',
				name = L["RX Check Delay"],
				desc = L["Set the delay before a Prescription check is run."],
				get = function() return self.db.profile.delay end,
				set = function(info, v) self.db.profile.delay = v end,
				min = 0,
				max = 60,
				step = 1,
			},
			drugs = {
				type = 'group',
				name = L["Prescription Drugs"],
				desc = L["What do you want to check for?"],
				args = {
					battle = {
						type = 'toggle',
						name = L["Battle Elixirs"],
						desc = L["Toggle checking for Battle Elixirs"],
						get = function() return self.db.profile.checkBattle end,
						set = function() self.db.profile.checkBattle = not self.db.profile.checkBattle end,
						order = 1,
					},
					guardian = {
						type = 'toggle',
						name = L["Guardian Elixirs"],
						desc = L["Toggle checking for Guardian Elixirs"],
						get = function() return self.db.profile.checkGuardian end,
						set = function() self.db.profile.checkGuardian = not self.db.profile.checkGuardian end,
						order = 2,
					},
					flask = {
						type = 'toggle',
						name = L["Flasks"],
						desc = L["Toggle checking for Flasks"],
						get = function() return self.db.profile.checkFlask end,
						set = function() self.db.profile.checkFlask = not self.db.profile.checkFlask end,
						order = 3,
					},
					food = {
						type = 'toggle',
						name = L["Food Buffs"],
						desc = L["Toggle checking for Food Buffs"],
						get = function() return self.db.profile.checkFood end,
						set = function() self.db.profile.checkFood = not self.db.profile.checkFood end,
						order = 4,
					},
					food250 = {
						type = 'toggle',
						name = L["Allow 250 Stat Food"],
						desc = L["Allows food buffs that only provide 250 stats to pass the food check"],
						get = function() return self.db.profile.allow250StatFood end,
						set = function() self.db.profile.allow250StatFood = not self.db.profile.allow250StatFood end,
						order = 6,
					},
					food275 = {
						type = 'toggle',
						name = L["Allow 275 Stat Food"],
						desc = L["Allows food buffs that only provide 275 stats to pass the food check"],
						get = function() return self.db.profile.allow275StatFood end,
						set = function() self.db.profile.allow275StatFood = not self.db.profile.allow275StatFood end,
						order = 7,
					},
				},
			},
			announce = {
				type = 'group',
				name = L["Announcements"],
				desc = L["How to announce Prescriptions"],
				args = {
					self = {
						type = 'toggle',
						name = L["Self Report"],
						desc = L["Print consumable report for yourself"],
						get = function() return self.db.profile.sendSelf end,
						set = function() self.db.profile.sendSelf = not self.db.profile.sendSelf end,
						order = 1,
					},
					whisper = {
						type = 'toggle',
						name = L["Whisper Players"],
						desc = L["Whisper players missing consumables"],
						get = function() return self.db.profile.sendWhispers end,
						set = function() self.db.profile.sendWhispers = not self.db.profile.sendWhispers end,
						order = 2,
					},
					blockouts = {
						type = 'toggle',
						name = L["Block Outgoing Whispers"],
						desc = L["Toggles blocking of outgoing <Rx> whispers"],
						get = function() return self.db.profile.blockOutgoing end,
						set = function() self.db.profile.blockOutgoing = not self.db.profile.blockOutgoing end,
						order = 3,
					},
					raid = {
						type = 'toggle',
						name = L["Raid Report"],
						desc = L["Send consumable reports to raid chat"],
						get = function() return self.db.profile.sendRaid end,
						set = function() self.db.profile.sendRaid = not self.db.profile.sendRaid end,
						order = 4,
					},
					officer = {
						type = 'toggle',
						name = L["Officer Report"],
						desc = L["Send consumable reports to officer chat"],
						get = function() return self.db.profile.sendOfficer end,
						set = function() self.db.profile.sendOfficer = not self.db.profile.sendOfficer end,
						order = 5,
					},
					channel = {
						type = 'toggle',
						name = L["Custom Channel Report"],
						desc = L["Send consumable reports to a user specified channel"],
						get = function() return self.db.profile.sendChannel end,
						set = function() self.db.profile.sendChannel = not self.db.profile.sendChannel end,
						order = 6,
					},
					channelName = {
						type = 'input',
						name = L["Custom Channel Name"],
						desc = L["Enter the name of the channel."],
						get = function() return self.db.profile.channel end,
						set = function(_, v) self.db.profile.channel = v end,
						disabled = function() return not self.db.profile.sendChannel end,
						order = 7,
					},
					compliance = {
						type = 'toggle',
						name = L["Compliance Broadcast"],
						desc = L["Broadcast the percent compliance to the raid after running a check."],
						get = function() return self.db.profile.sendCompliance end,
						set = function() self.db.profile.sendCompliance = not self.db.profile.sendCompliance end,
						order = 8,
					},
					hide = {
						type = 'toggle',
						name = L["Hide \"Time to Check\" Alert Message"],
						desc = L["Hide the message that alerts the raid to an incoming RX Check"],
						get = function() return self.db.profile.hideAlert end,
						set = function() self.db.profile.hideAlert = not self.db.profile.hideAlert end,
						order = 9,
					},
				},
			},
		},
	}
	opts.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	--register the config
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Prescription", opts)
	LibStub("AceConfigDialog-3.0"):SetDefaultSize("Prescription", 500, 550)
	
	--register slash commands
	_G["SlashCmdList"]["PRESCRIPTION_MAIN"] = function(s)
		if strlower(s) == "check" then
			if not IsInRaid() then
				self:Print(format("|cffff0000%s|r", L["You are not in a raid!"]))
				return
			end
			self:ScheduleTimer("Raid_Check", self.db.profile.delay)
			if not self.db.profile.hideAlert then
				if UnitIsGroupLeader("player") or UnitIsGroupAssistant("player") then
					SendChatMessage(format(L["RX Check will be run in %d seconds!"], self.db.profile.delay), "RAID_WARNING")
				else
					SendChatMessage(format(L["RX Check will be run in %d seconds!"], self.db.profile.delay), "RAID")
				end
			end
		elseif strlower(s) == "instant" then
			if not IsInRaid() then
				self:Print(format("|cffff0000%s|r", L["You are not in a raid!"]))
				return
			end
			if not self.db.profile.hideAlert then
				if UnitIsGroupLeader("player") or UnitIsGroupAssistant("player") then
					SendChatMessage(L["RX Check running now!"], "RAID_WARNING")
				else
					SendChatMessage(L["RX Check running now!"], "RAID")
				end
			end
			self:Raid_Check()
		else
			LibStub("AceConfigDialog-3.0"):Open("Prescription")
		end
	end
	_G["SlashCmdList"]["PRESCRIPTION_CHECK"] = function()
		if not IsInRaid() then
			self:Print(format("|cffff0000%s|r", L["You are not in a raid!"]))
			return
		end
		self:ScheduleTimer("Raid_Check", self.db.profile.delay)
		if not self.db.profile.hideAlert then
			if UnitIsGroupLeader("player") or UnitIsGroupAssistant("player") then
				SendChatMessage(format(L["RX Check will be run in %d seconds!"], self.db.profile.delay), "RAID_WARNING")
			else
				SendChatMessage(format(L["RX Check will be run in %d seconds!"], self.db.profile.delay), "RAID")
			end
		end
	end
	_G["SLASH_PRESCRIPTION_MAIN1"] = "/rx"
	_G["SLASH_PRESCRIPTION_MAIN2"] = "/prescription"
	_G["SLASH_PRESCRIPTION_CHECK1"] = "/rxc"
	
	--message filter for blocking outgoing RX whisper reports
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", function(self, event, ...)
		local msg = ...
		if Prescription.db.profile.blockOutgoing and msg:find("^<RX> ") and UnitInRaid("player") then
			return true, ...
		end
		return false, ...
	end)
end

function Prescription:Raid_Check()
	if not IsInRaid() then
		self:Print(format("|cffff0000%s|r", L["You are not in a raid!"]))
		return
	end
	if InCombatLockdown() then
		self:Print(format("|cffff0000%s|r", L["Combat produces irregular results, aborting check."]))
		return
	end
	
	local numRaid = GetNumGroupMembers()
	local processed = 0
	local compliant = 0
	local messages = {}
	
	for i = 1, numRaid do
		if UnitIsConnected("raid"..i) and UnitIsVisible("raid"..i) and (not UnitIsDead("raid"..i)) then
			processed = processed + 1
			
			local hasFlask, hasGuardian, hasBattle, hasFood = false, false, false, false
			
			if UnitBuff("raid"..i, 1) ~= nil then
				--raider has buffs, start checking
				local n = 1
				while UnitBuff("raid"..i, n) ~= nil do
					--get the spellId of the buff
					local id = select(11, UnitBuff("raid"..i, n))
					
					--check
					if flaskList[id] then
						--flask check passed
						hasFlask = true
					end
					
					if battleList[id] then
						--battle elixir check passed
						hasBattle = true
					end
					
					if guardianList[id] then
						--guardian elixir check passed
						hasGuardian = true
					end
					
					if foodList[id] or (self.db.profile.allow250StatFood and foodList250[id]) or (self.db.profile.allow275StatFood and foodList275[id]) then
						--food checks pass
						hasFood = true
					end
					
					--update counter
					n = n + 1
				end
			end
			
			--if we're flasked, we're elixir'd too
			if hasFlask then hasBattle, hasGuardian = true, true end
			
			--if we have 2 elixirs, count as flasked?
			if hasBattle and hasGuardian then hasFlask = true end
			
			local message = {}
			--now process the flags
			if self.db.profile.checkFlask and not hasFlask then
				message[#message+1] = L["Flask"]
			end
			if self.db.profile.checkBattle and not hasBattle then
				message[#message+1] = L["Battle Elixir"]
			end
			if self.db.profile.checkGuardian and not hasGuardian then
				message[#message+1] = L["Guardian Elixir"]
			end
			if self.db.profile.checkFood and not hasFood then
				message[#message+1] = L["Food"]
			end
			
			if #message == 0 then
				compliant = compliant + 1
			else
				messages[UnitName("raid"..i)] = table.concat(message, ", ")
			end
		end
	end
	
	--send individual whispers out first
	if self.db.profile.sendWhispers then
		for name, list in pairs(messages) do
			 if (name ~= user) then
				SendChatMessage(format("<RX> %s%s", L["You are missing: "], list), "WHISPER", nil, name)
			end
		end
	end
	
	if self.db.profile.sendRaid then
		for name, list in pairs(messages) do
			local prefix = format(L["%s is missing: "], name)
			SendChatMessage(format("<RX> %s%s", prefix, list), "RAID")
		end
	end
	if self.db.profile.sendOfficer then
		for name, list in pairs(messages) do
			local prefix = format(L["%s is missing: "], name)
			SendChatMessage(format("<RX> %s%s", prefix, list), "OFFICER")
		end
	end
	if self.db.profile.sendSelf then
		for name, list in pairs(messages) do
			local prefix = format(L["%s is missing: "], name)
			self:Print(format("<RX> %s%s", prefix, list))
		end
	end
	if self.db.profile.sendChannel then
		for name, list in pairs(messages) do
			local prefix = format(L["%s is missing: "], name)
			SendChatMessage(format("<RX> %s%s", prefix, list), "CHANNEL", nil, GetChannelName(self.db.profile.channel))
		end
	end
	
	--print a compliance percent if enabled
	if self.db.profile.sendCompliance then
		if compliant == processed then
			SendChatMessage("<RX> " .. L["100% Compliance, Thank You."], "RAID")
		else
			SendChatMessage("<RX> " .. format(L["%d%% Compliance."], compliant / processed * 100), "RAID")
		end
	end
end
